@@ -9,14 +9,26 @@
 #include "wlr-layer-shell-unstable-v1-client-protocol.h"
 #include "xdg-output-unstable-v1-client-protocol.h"
 
+#define TOUCH_ID_EMPTY -1
+
 struct slurp_box {
 	int32_t x, y;
 	int32_t width, height;
+	char *label;
 	struct wl_list link;
 };
 
+struct slurp_selection {
+	struct slurp_output *current_output;
+	int32_t x, y;
+	int32_t anchor_x, anchor_y;
+	struct slurp_box selection;
+	bool has_selection;
+};
+
 struct slurp_state {
 	bool running;
+	bool edit_anchor;
 
 	struct wl_display *display;
 	struct wl_registry *registry;
@@ -27,15 +39,19 @@ struct slurp_state {
 	struct wl_list outputs; // slurp_output::link
 	struct wl_list seats; // slurp_seat::link
 
+	struct xkb_context *xkb_context;
+
 	struct {
 		uint32_t background;
 		uint32_t border;
 		uint32_t selection;
+		uint32_t choice;
 	} colors;
 
 	uint32_t border_weight;
 	bool display_dimensions;
 	bool single_point;
+	bool restrict_selection;
 	struct wl_list boxes; // slurp_box::link
 
 	struct slurp_box result;
@@ -75,14 +91,22 @@ struct slurp_seat {
 	// keyboard:
 	struct wl_keyboard *wl_keyboard;
 
+	//selection (pointer/touch):
+
+	struct slurp_selection pointer_selection;
+	struct slurp_selection touch_selection;
+
 	// pointer:
 	struct wl_pointer *wl_pointer;
 	enum wl_pointer_button_state button_state;
-	struct slurp_output *current_output;
-	int32_t x, y;
-	int32_t pressed_x, pressed_y;
-	struct slurp_box selection;
-	bool has_selection;
+
+	// keymap:
+	struct xkb_keymap *xkb_keymap;
+	struct xkb_state *xkb_state;
+
+	// touch:
+	struct wl_touch *wl_touch;
+  	int32_t touch_id;
 };
 
 bool box_intersect(const struct slurp_box *a, const struct slurp_box *b);
